"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 29a56c8fab24

from .basesdk import BaseSDK
from mollie import models, utils
from mollie._hooks import HookContext
from mollie.types import Nullable, OptionalNullable, UNSET
from mollie.utils import get_security_from_env
from mollie.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class Methods(BaseSDK):
    def list(
        self,
        *,
        sequence_type: Optional[models.SequenceType] = None,
        locale: OptionalNullable[models.Locale] = UNSET,
        amount: Optional[Union[models.Amount, models.AmountTypedDict]] = None,
        resource: Optional[models.MethodResourceParameter] = None,
        billing_country: Optional[str] = None,
        include_wallets: Optional[models.MethodIncludeWalletsParameter] = None,
        order_line_categories: Optional[models.LineCategories] = None,
        profile_id: Optional[str] = None,
        include: OptionalNullable[str] = UNSET,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListMethodsResponse:
        r"""List payment methods

        Retrieve all enabled payment methods. The results of this endpoint are
        **not** paginated — unlike most other list endpoints in our API.

        For test mode, all pending and enabled payment methods are returned. If no
        payment methods are requested yet, the most popular payment methods are returned in the test mode. For live
        mode, only fully enabled payment methods are returned.

        Payment methods can be requested and enabled via the Mollie Dashboard, or
        via the [Enable payment method endpoint](enable-method) of the Profiles API.

        The list can optionally be filtered using a number of parameters described
        below.

        By default, only payment methods for the Euro currency are returned. If you
        wish to retrieve payment methods which exclusively support other currencies (e.g. Twint), you need to use the
        `amount` parameters.

        ℹ️ **Note:** This endpoint only returns **online** payment methods. If you wish to retrieve the information about
        a non-online payment method, you can use the [Get payment method endpoint](get-method).

        :param sequence_type: Set this parameter to `first` to only return the enabled methods that
            can be used for the first payment of a recurring sequence.

            Set it to `recurring` to only return enabled methods that can be used for recurring payments or subscriptions.
        :param locale: Passing a locale will sort the payment methods in the preferred order
            for the country, and translate the payment method names in the corresponding language.
        :param amount: If supplied, only payment methods that support the amount and currency
            are returned.

            Example: `/v2/methods?amount[value]=100.00&amount[currency]=USD`
        :param resource: **⚠️ We no longer recommend using the Orders API. Please refer to the [Payments API](payments-api) instead.**

            Indicate if you will use the result for the [Create order](create-order)
            or the [Create payment](create-payment) endpoint.

            When passing the value `orders`, the result will include payment methods
            that are only available for payments created via the Orders API.
        :param billing_country: The country taken from your customer's billing address in ISO 3166-1 alpha-2 format. This parameter can be used
            to check whether your customer is eligible for certain payment methods, for example for Klarna.

            Example: `/v2/methods?resource=orders&billingCountry=DE`
        :param include_wallets: A comma-separated list of the wallets you support in your checkout. Wallets often require wallet specific code
            to check if they are available on the shoppers device, hence the need to indicate your support.
        :param order_line_categories: A comma-separated list of the line categories you support in your checkout.

            Example: `/v2/methods?orderLineCategories=eco,meal`
        :param profile_id: The identifier referring to the [profile](get-profile) you wish to
            retrieve the resources for.

            Most API credentials are linked to a single profile. In these cases the `profileId` must not be sent. For
            organization-level credentials such as OAuth access tokens however, the `profileId` parameter is required.
        :param include: This endpoint allows you to include additional information via the
            `include` query string parameter.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListMethodsRequest(
            sequence_type=sequence_type,
            locale=locale,
            amount=utils.get_pydantic_model(amount, Optional[models.Amount]),
            resource=resource,
            billing_country=billing_country,
            include_wallets=include_wallets,
            order_line_categories=order_line_categories,
            profile_id=profile_id,
            include=include,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/methods",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.ListMethodsGlobals(
                profile_id=self.sdk_configuration.globals.profile_id,
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list-methods",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.ListMethodsResponse, http_res)
        if utils.match_response(http_res, "400", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        sequence_type: Optional[models.SequenceType] = None,
        locale: OptionalNullable[models.Locale] = UNSET,
        amount: Optional[Union[models.Amount, models.AmountTypedDict]] = None,
        resource: Optional[models.MethodResourceParameter] = None,
        billing_country: Optional[str] = None,
        include_wallets: Optional[models.MethodIncludeWalletsParameter] = None,
        order_line_categories: Optional[models.LineCategories] = None,
        profile_id: Optional[str] = None,
        include: OptionalNullable[str] = UNSET,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListMethodsResponse:
        r"""List payment methods

        Retrieve all enabled payment methods. The results of this endpoint are
        **not** paginated — unlike most other list endpoints in our API.

        For test mode, all pending and enabled payment methods are returned. If no
        payment methods are requested yet, the most popular payment methods are returned in the test mode. For live
        mode, only fully enabled payment methods are returned.

        Payment methods can be requested and enabled via the Mollie Dashboard, or
        via the [Enable payment method endpoint](enable-method) of the Profiles API.

        The list can optionally be filtered using a number of parameters described
        below.

        By default, only payment methods for the Euro currency are returned. If you
        wish to retrieve payment methods which exclusively support other currencies (e.g. Twint), you need to use the
        `amount` parameters.

        ℹ️ **Note:** This endpoint only returns **online** payment methods. If you wish to retrieve the information about
        a non-online payment method, you can use the [Get payment method endpoint](get-method).

        :param sequence_type: Set this parameter to `first` to only return the enabled methods that
            can be used for the first payment of a recurring sequence.

            Set it to `recurring` to only return enabled methods that can be used for recurring payments or subscriptions.
        :param locale: Passing a locale will sort the payment methods in the preferred order
            for the country, and translate the payment method names in the corresponding language.
        :param amount: If supplied, only payment methods that support the amount and currency
            are returned.

            Example: `/v2/methods?amount[value]=100.00&amount[currency]=USD`
        :param resource: **⚠️ We no longer recommend using the Orders API. Please refer to the [Payments API](payments-api) instead.**

            Indicate if you will use the result for the [Create order](create-order)
            or the [Create payment](create-payment) endpoint.

            When passing the value `orders`, the result will include payment methods
            that are only available for payments created via the Orders API.
        :param billing_country: The country taken from your customer's billing address in ISO 3166-1 alpha-2 format. This parameter can be used
            to check whether your customer is eligible for certain payment methods, for example for Klarna.

            Example: `/v2/methods?resource=orders&billingCountry=DE`
        :param include_wallets: A comma-separated list of the wallets you support in your checkout. Wallets often require wallet specific code
            to check if they are available on the shoppers device, hence the need to indicate your support.
        :param order_line_categories: A comma-separated list of the line categories you support in your checkout.

            Example: `/v2/methods?orderLineCategories=eco,meal`
        :param profile_id: The identifier referring to the [profile](get-profile) you wish to
            retrieve the resources for.

            Most API credentials are linked to a single profile. In these cases the `profileId` must not be sent. For
            organization-level credentials such as OAuth access tokens however, the `profileId` parameter is required.
        :param include: This endpoint allows you to include additional information via the
            `include` query string parameter.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListMethodsRequest(
            sequence_type=sequence_type,
            locale=locale,
            amount=utils.get_pydantic_model(amount, Optional[models.Amount]),
            resource=resource,
            billing_country=billing_country,
            include_wallets=include_wallets,
            order_line_categories=order_line_categories,
            profile_id=profile_id,
            include=include,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/methods",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.ListMethodsGlobals(
                profile_id=self.sdk_configuration.globals.profile_id,
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list-methods",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.ListMethodsResponse, http_res)
        if utils.match_response(http_res, "400", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def all(
        self,
        *,
        locale: OptionalNullable[models.Locale] = UNSET,
        amount: Optional[Union[models.Amount, models.AmountTypedDict]] = None,
        include: OptionalNullable[str] = UNSET,
        sequence_type: Optional[models.SequenceType] = None,
        profile_id: Optional[str] = None,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListAllMethodsResponse:
        r"""List all payment methods

        Retrieve all payment methods that Mollie offers, regardless of the eligibility of the organization for the specific
        method. The results of this endpoint are **not** paginated — unlike most other list endpoints in our API.

        The list can optionally be filtered using a number of parameters described below.

        ℹ️ **Note:** This endpoint only returns **online** payment methods. If you wish to retrieve the information about
        a non-online payment method, you can use the [Get payment method endpoint](get-method).

        :param locale: Passing a locale will sort the payment methods in the preferred order
            for the country, and translate the payment method names in the corresponding language.
        :param amount: If supplied, only payment methods that support the amount and currency
            are returned.

            Example: `/v2/methods/all?amount[value]=100.00&amount[currency]=USD`
        :param include: This endpoint allows you to include additional information via the
            `include` query string parameter.
        :param sequence_type: Set this parameter to `first` to only return the methods that
            can be used for the first payment of a recurring sequence.

            Set it to `recurring` to only return methods that can be used for recurring payments or subscriptions.
        :param profile_id: The identifier referring to the [profile](get-profile) you wish to
            retrieve the resources for.

            Most API credentials are linked to a single profile. In these cases the `profileId` must not be sent. For
            organization-level credentials such as OAuth access tokens however, the `profileId` parameter is required.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListAllMethodsRequest(
            locale=locale,
            amount=utils.get_pydantic_model(amount, Optional[models.Amount]),
            include=include,
            sequence_type=sequence_type,
            profile_id=profile_id,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/methods/all",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.ListAllMethodsGlobals(
                profile_id=self.sdk_configuration.globals.profile_id,
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list-all-methods",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.ListAllMethodsResponse, http_res)
        if utils.match_response(http_res, "400", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def all_async(
        self,
        *,
        locale: OptionalNullable[models.Locale] = UNSET,
        amount: Optional[Union[models.Amount, models.AmountTypedDict]] = None,
        include: OptionalNullable[str] = UNSET,
        sequence_type: Optional[models.SequenceType] = None,
        profile_id: Optional[str] = None,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListAllMethodsResponse:
        r"""List all payment methods

        Retrieve all payment methods that Mollie offers, regardless of the eligibility of the organization for the specific
        method. The results of this endpoint are **not** paginated — unlike most other list endpoints in our API.

        The list can optionally be filtered using a number of parameters described below.

        ℹ️ **Note:** This endpoint only returns **online** payment methods. If you wish to retrieve the information about
        a non-online payment method, you can use the [Get payment method endpoint](get-method).

        :param locale: Passing a locale will sort the payment methods in the preferred order
            for the country, and translate the payment method names in the corresponding language.
        :param amount: If supplied, only payment methods that support the amount and currency
            are returned.

            Example: `/v2/methods/all?amount[value]=100.00&amount[currency]=USD`
        :param include: This endpoint allows you to include additional information via the
            `include` query string parameter.
        :param sequence_type: Set this parameter to `first` to only return the methods that
            can be used for the first payment of a recurring sequence.

            Set it to `recurring` to only return methods that can be used for recurring payments or subscriptions.
        :param profile_id: The identifier referring to the [profile](get-profile) you wish to
            retrieve the resources for.

            Most API credentials are linked to a single profile. In these cases the `profileId` must not be sent. For
            organization-level credentials such as OAuth access tokens however, the `profileId` parameter is required.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListAllMethodsRequest(
            locale=locale,
            amount=utils.get_pydantic_model(amount, Optional[models.Amount]),
            include=include,
            sequence_type=sequence_type,
            profile_id=profile_id,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/methods/all",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.ListAllMethodsGlobals(
                profile_id=self.sdk_configuration.globals.profile_id,
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list-all-methods",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.ListAllMethodsResponse, http_res)
        if utils.match_response(http_res, "400", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        method_id: Nullable[models.MethodEnum],
        locale: OptionalNullable[models.Locale] = UNSET,
        currency: Optional[str] = None,
        profile_id: Optional[str] = None,
        include: OptionalNullable[str] = UNSET,
        sequence_type: Optional[models.SequenceType] = None,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EntityMethodGet:
        r"""Get payment method

        Retrieve a single payment method by its ID.

        If a method is not available on this profile, a `404 Not Found` response is
        returned. If the method is available but not enabled yet, a status `403 Forbidden` is returned. You can enable
        payments methods via the [Enable payment method endpoint](enable-method) of the Profiles API, or via
        the Mollie Dashboard.

        If you do not know the method's ID, you can use the [methods list
        endpoint](list-methods) to retrieve all payment methods that are available.

        Additionally, it is possible to check if wallet methods such as Apple Pay
        are enabled by passing the wallet ID (`applepay`) as the method ID.

        :param method_id: Provide the ID of the related payment method.
        :param locale: Passing a locale will sort the payment methods in the preferred order
            for the country, and translate the payment method names in the corresponding language.
        :param currency: If provided, the `minimumAmount` and `maximumAmount` will be converted
            to the given currency. An error is returned if the currency is not supported by the payment method.
        :param profile_id: The identifier referring to the [profile](get-profile) you wish to
            retrieve the resources for.

            Most API credentials are linked to a single profile. In these cases the `profileId` must not be sent. For
            organization-level credentials such as OAuth access tokens however, the `profileId` parameter is required.
        :param include: This endpoint allows you to include additional information via the
            `include` query string parameter.
        :param sequence_type: Set this parameter to `first` to only return the methods that
            can be used for the first payment of a recurring sequence.

            Set it to `recurring` to only return methods that can be used for recurring payments or subscriptions.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetMethodRequest(
            method_id=method_id,
            locale=locale,
            currency=currency,
            profile_id=profile_id,
            include=include,
            sequence_type=sequence_type,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/methods/{methodId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.GetMethodGlobals(
                profile_id=self.sdk_configuration.globals.profile_id,
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get-method",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.EntityMethodGet, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        method_id: Nullable[models.MethodEnum],
        locale: OptionalNullable[models.Locale] = UNSET,
        currency: Optional[str] = None,
        profile_id: Optional[str] = None,
        include: OptionalNullable[str] = UNSET,
        sequence_type: Optional[models.SequenceType] = None,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EntityMethodGet:
        r"""Get payment method

        Retrieve a single payment method by its ID.

        If a method is not available on this profile, a `404 Not Found` response is
        returned. If the method is available but not enabled yet, a status `403 Forbidden` is returned. You can enable
        payments methods via the [Enable payment method endpoint](enable-method) of the Profiles API, or via
        the Mollie Dashboard.

        If you do not know the method's ID, you can use the [methods list
        endpoint](list-methods) to retrieve all payment methods that are available.

        Additionally, it is possible to check if wallet methods such as Apple Pay
        are enabled by passing the wallet ID (`applepay`) as the method ID.

        :param method_id: Provide the ID of the related payment method.
        :param locale: Passing a locale will sort the payment methods in the preferred order
            for the country, and translate the payment method names in the corresponding language.
        :param currency: If provided, the `minimumAmount` and `maximumAmount` will be converted
            to the given currency. An error is returned if the currency is not supported by the payment method.
        :param profile_id: The identifier referring to the [profile](get-profile) you wish to
            retrieve the resources for.

            Most API credentials are linked to a single profile. In these cases the `profileId` must not be sent. For
            organization-level credentials such as OAuth access tokens however, the `profileId` parameter is required.
        :param include: This endpoint allows you to include additional information via the
            `include` query string parameter.
        :param sequence_type: Set this parameter to `first` to only return the methods that
            can be used for the first payment of a recurring sequence.

            Set it to `recurring` to only return methods that can be used for recurring payments or subscriptions.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetMethodRequest(
            method_id=method_id,
            locale=locale,
            currency=currency,
            profile_id=profile_id,
            include=include,
            sequence_type=sequence_type,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/methods/{methodId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.GetMethodGlobals(
                profile_id=self.sdk_configuration.globals.profile_id,
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get-method",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.EntityMethodGet, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f9ec93f73c31

from __future__ import annotations
from enum import Enum
from mollie import utils


class PaymentDetailsReceiptCardVerificationMethodResponse(
    str, Enum, metaclass=utils.OpenEnumMeta
):
    r"""The method used to verify the cardholder's identity."""

    NO_CVM_REQUIRED = "no-cvm-required"
    ONLINE_PIN = "online-pin"
    OFFLINE_PIN = "offline-pin"
    CONSUMER_DEVICE = "consumer-device"
    SIGNATURE = "signature"
    SIGNATURE_AND_ONLINE_PIN = "signature-and-online-pin"
    ONLINE_PIN_AND_SIGNATURE = "online-pin-and-signature"
    NONE = "none"
    FAILED = "failed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 21046be972d4

from __future__ import annotations
from .entity_balance_transfer import (
    EntityBalanceTransfer,
    EntityBalanceTransferTypedDict,
)
from mollie.types import BaseModel, UNSET_SENTINEL
from mollie.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateConnectBalanceTransferRequestTypedDict(TypedDict):
    idempotency_key: NotRequired[str]
    r"""A unique key to ensure idempotent requests. This key should be a UUID v4 string."""
    entity_balance_transfer: NotRequired[EntityBalanceTransferTypedDict]


class CreateConnectBalanceTransferRequest(BaseModel):
    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A unique key to ensure idempotent requests. This key should be a UUID v4 string."""

    entity_balance_transfer: Annotated[
        Optional[EntityBalanceTransfer],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["idempotency-key", "entity-balance-transfer"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f36b974e58fc

from __future__ import annotations
from mollie.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class URLNullableTypedDict(TypedDict):
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    href: NotRequired[str]
    r"""The actual URL string."""
    type: NotRequired[str]
    r"""The content type of the page or endpoint the URL points to."""


class URLNullable(BaseModel):
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    href: Optional[str] = None
    r"""The actual URL string."""

    type: Optional[str] = None
    r"""The content type of the page or endpoint the URL points to."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["href", "type"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6fe107b20f69

from __future__ import annotations
from .mode import Mode
from .profile_review_status_response import ProfileReviewStatusResponse
from .url import URL, URLTypedDict
from enum import Enum
from mollie import models, utils
from mollie.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListProfileResponseStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The profile status determines whether the profile is able to receive live payments.

    * `unverified`: The profile has not been verified yet and can only be used to create test payments.
    * `verified`: The profile has been verified and can be used to create live payments and test payments.
    * `blocked`: The profile is blocked and can no longer be used or changed.
    """

    UNVERIFIED = "unverified"
    VERIFIED = "verified"
    BLOCKED = "blocked"


class ListProfileResponseReviewTypedDict(TypedDict):
    r"""Present if changes have been made that have not yet been approved by Mollie. Changes to test profiles are approved
    automatically, unless a switch to a live profile has been requested. The review object will therefore usually be
    `null` in test mode.
    """

    status: NotRequired[ProfileReviewStatusResponse]
    r"""The status of the requested changes."""


class ListProfileResponseReview(BaseModel):
    r"""Present if changes have been made that have not yet been approved by Mollie. Changes to test profiles are approved
    automatically, unless a switch to a live profile has been requested. The review object will therefore usually be
    `null` in test mode.
    """

    status: Optional[ProfileReviewStatusResponse] = None
    r"""The status of the requested changes."""

    @field_serializer("status")
    def serialize_status(self, value):
        if isinstance(value, str):
            try:
                return models.ProfileReviewStatusResponse(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["status"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListProfileResponseLinksTypedDict(TypedDict):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    dashboard: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    chargebacks: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    methods: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    payments: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    refunds: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    checkout_preview_url: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""


class ListProfileResponseLinks(BaseModel):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: Annotated[Optional[URL], pydantic.Field(alias="self")] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    dashboard: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    chargebacks: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    methods: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    payments: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    refunds: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    checkout_preview_url: Annotated[
        Optional[URL], pydantic.Field(alias="checkoutPreviewUrl")
    ] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "self",
                "dashboard",
                "chargebacks",
                "methods",
                "payments",
                "refunds",
                "checkoutPreviewUrl",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListProfileResponseTypedDict(TypedDict):
    resource: str
    r"""Indicates the response contains a profile object. Will always contain the string `profile` for this endpoint."""
    id: str
    r"""The identifier uniquely referring to this profile. Example: `pfl_v9hTwCvYqw`."""
    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""
    name: str
    r"""The profile's name, this will usually reflect the trade name or brand name of the profile's website or
    application.
    """
    website: str
    r"""The URL to the profile's website or application. Only `https` or `http` URLs are allowed. No `@` signs are
    allowed.
    """
    email: str
    r"""The email address associated with the profile's trade name or brand."""
    phone: str
    r"""The phone number associated with the profile's trade name or brand."""
    business_category: str
    r"""The industry associated with the profile's trade name or brand. Please refer to the
    [business category list](common-data-types#business-category) for all possible options.
    """
    status: ListProfileResponseStatus
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""
    links: ListProfileResponseLinksTypedDict
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""
    description: NotRequired[str]
    r"""The products or services offered by the profile's website or application."""
    countries_of_activity: NotRequired[List[str]]
    r"""A list of countries where you expect that the majority of the profile's customers reside,
    in [ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) format.
    """
    review: NotRequired[ListProfileResponseReviewTypedDict]
    r"""Present if changes have been made that have not yet been approved by Mollie. Changes to test profiles are approved
    automatically, unless a switch to a live profile has been requested. The review object will therefore usually be
    `null` in test mode.
    """


class ListProfileResponse(BaseModel):
    resource: str
    r"""Indicates the response contains a profile object. Will always contain the string `profile` for this endpoint."""

    id: str
    r"""The identifier uniquely referring to this profile. Example: `pfl_v9hTwCvYqw`."""

    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""

    name: str
    r"""The profile's name, this will usually reflect the trade name or brand name of the profile's website or
    application.
    """

    website: str
    r"""The URL to the profile's website or application. Only `https` or `http` URLs are allowed. No `@` signs are
    allowed.
    """

    email: str
    r"""The email address associated with the profile's trade name or brand."""

    phone: str
    r"""The phone number associated with the profile's trade name or brand."""

    business_category: Annotated[str, pydantic.Field(alias="businessCategory")]
    r"""The industry associated with the profile's trade name or brand. Please refer to the
    [business category list](common-data-types#business-category) for all possible options.
    """

    status: ListProfileResponseStatus

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""

    links: Annotated[ListProfileResponseLinks, pydantic.Field(alias="_links")]
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    description: Optional[str] = None
    r"""The products or services offered by the profile's website or application."""

    countries_of_activity: Annotated[
        Optional[List[str]], pydantic.Field(alias="countriesOfActivity")
    ] = None
    r"""A list of countries where you expect that the majority of the profile's customers reside,
    in [ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) format.
    """

    review: Optional[ListProfileResponseReview] = None
    r"""Present if changes have been made that have not yet been approved by Mollie. Changes to test profiles are approved
    automatically, unless a switch to a live profile has been requested. The review object will therefore usually be
    `null` in test mode.
    """

    @field_serializer("mode")
    def serialize_mode(self, value):
        if isinstance(value, str):
            try:
                return models.Mode(value)
            except ValueError:
                return value
        return value

    @field_serializer("status")
    def serialize_status(self, value):
        if isinstance(value, str):
            try:
                return models.ListProfileResponseStatus(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "countriesOfActivity", "review"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ListProfileResponseLinks.model_rebuild()
except NameError:
    pass
try:
    ListProfileResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 79446cf808c7

from __future__ import annotations
from enum import Enum
from mollie import utils


class BalanceFeeType(str, Enum, metaclass=utils.OpenEnumMeta):
    PAYMENT_FEE = "payment-fee"
    DIRECT_DEBIT_FAILURE_FEE = "direct-debit-failure-fee"
    UNAUTHORIZED_DIRECT_DEBIT_FEE = "unauthorized-direct-debit-fee"
    BANK_CHARGED_DIRECT_DEBIT_FAILURE_FEE = "bank-charged-direct-debit-failure-fee"
    PARTNER_COMMISSION = "partner-commission"
    APPLICATION_FEE = "application-fee"
    CAPTURE_FEE = "capture-fee"
    REFUND_FEE = "refund-fee"
    CHARGEBACK_FEE = "chargeback-fee"
    PAYMENT_NOTIFICATION_FEE = "payment-notification-fee"
    TRANSFER_NOTIFICATION_FEE = "transfer-notification-fee"
    PAYOUT_FEE = "payout-fee"
    FEE_DISCOUNT = "fee-discount"
    FEE_REIMBURSEMENT = "fee-reimbursement"
    PLATFORM_VOLUME_FEE = "platform-volume-fee"
    PLATFORM_CONNECTED_ORGANIZATIONS_FEE = "platform-connected-organizations-fee"
    BALANCE_CHARGE_FEE = "balance-charge-fee"
    THREEDS_AUTHENTICATION_ATTEMPT_FEE = "3ds-authentication-attempt-fee"
    TERMINAL_MONTHLY_FEE = "terminal-monthly-fee"
    ACCEPTANCE_RISK_FEE = "acceptance-risk-fee"
    TOP_UP_FEE = "top-up-fee"
    PAYMENT_GATEWAY_FEE = "payment-gateway-fee"
    MASTERCARD_SPECIALTY_MERCHANT_PROGRAM_PROCESSING_FEE = (
        "mastercard-specialty-merchant-program-processing-fee"
    )
    MASTERCARD_SPECIALTY_MERCHANT_PROGRAM_REGISTRATION_FEE = (
        "mastercard-specialty-merchant-program-registration-fee"
    )
    VISA_INTEGRITY_RISK_PROGRAM_PROCESSING_FEE = (
        "visa-integrity-risk-program-processing-fee"
    )
    VISA_INTEGRITY_RISK_PROGRAM_REGISTRATION_FEE = (
        "visa-integrity-risk-program-registration-fee"
    )
    MINIMUM_INVOICE_AMOUNT_FEE = "minimum-invoice-amount-fee"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f5bce20d2965

from __future__ import annotations
from .mandate_details_card_label_response import MandateDetailsCardLabelResponse
from .mandate_method_response import MandateMethodResponse
from .mode import Mode
from .url import URL, URLTypedDict
from enum import Enum
from mollie import models, utils
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class ListMandateResponseDetailsTypedDict(TypedDict):
    consumer_name: NotRequired[Nullable[str]]
    r"""The customer's name. Available for SEPA Direct Debit and PayPal mandates."""
    consumer_account: NotRequired[Nullable[str]]
    r"""The customer's IBAN or email address. Available for SEPA Direct Debit and PayPal mandates."""
    consumer_bic: NotRequired[Nullable[str]]
    r"""The BIC of the customer's bank. Available for SEPA Direct Debit mandates."""
    card_holder: NotRequired[Nullable[str]]
    r"""The card holder's name. Available for card mandates."""
    card_number: NotRequired[Nullable[str]]
    r"""The last four digits of the card number. Available for card mandates."""
    card_expiry_date: NotRequired[Nullable[str]]
    r"""The card's expiry date in `YYYY-MM-DD` format. Available for card mandates."""
    card_label: NotRequired[Nullable[MandateDetailsCardLabelResponse]]
    r"""The card's label. Available for card mandates, if the card label could be detected."""
    card_fingerprint: NotRequired[Nullable[str]]
    r"""Unique alphanumeric representation of this specific card. Available for card mandates. Can be used to identify
    returning customers.
    """


class ListMandateResponseDetails(BaseModel):
    consumer_name: Annotated[
        OptionalNullable[str], pydantic.Field(alias="consumerName")
    ] = UNSET
    r"""The customer's name. Available for SEPA Direct Debit and PayPal mandates."""

    consumer_account: Annotated[
        OptionalNullable[str], pydantic.Field(alias="consumerAccount")
    ] = UNSET
    r"""The customer's IBAN or email address. Available for SEPA Direct Debit and PayPal mandates."""

    consumer_bic: Annotated[
        OptionalNullable[str], pydantic.Field(alias="consumerBic")
    ] = UNSET
    r"""The BIC of the customer's bank. Available for SEPA Direct Debit mandates."""

    card_holder: Annotated[
        OptionalNullable[str], pydantic.Field(alias="cardHolder")
    ] = UNSET
    r"""The card holder's name. Available for card mandates."""

    card_number: Annotated[
        OptionalNullable[str], pydantic.Field(alias="cardNumber")
    ] = UNSET
    r"""The last four digits of the card number. Available for card mandates."""

    card_expiry_date: Annotated[
        OptionalNullable[str], pydantic.Field(alias="cardExpiryDate")
    ] = UNSET
    r"""The card's expiry date in `YYYY-MM-DD` format. Available for card mandates."""

    card_label: Annotated[
        OptionalNullable[MandateDetailsCardLabelResponse],
        pydantic.Field(alias="cardLabel"),
    ] = UNSET
    r"""The card's label. Available for card mandates, if the card label could be detected."""

    card_fingerprint: Annotated[
        OptionalNullable[str], pydantic.Field(alias="cardFingerprint")
    ] = UNSET
    r"""Unique alphanumeric representation of this specific card. Available for card mandates. Can be used to identify
    returning customers.
    """

    @field_serializer("card_label")
    def serialize_card_label(self, value):
        if isinstance(value, str):
            try:
                return models.MandateDetailsCardLabelResponse(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "consumerName",
                "consumerAccount",
                "consumerBic",
                "cardHolder",
                "cardNumber",
                "cardExpiryDate",
                "cardLabel",
                "cardFingerprint",
            ]
        )
        nullable_fields = set(
            [
                "consumerName",
                "consumerAccount",
                "consumerBic",
                "cardHolder",
                "cardNumber",
                "cardExpiryDate",
                "cardLabel",
                "cardFingerprint",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ListMandateResponseStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The status of the mandate. A status can be `pending` for mandates when the first payment is not yet finalized, or
    when we did not received the IBAN yet from the first payment.
    """

    VALID = "valid"
    PENDING = "pending"
    INVALID = "invalid"


class ListMandateResponseLinksTypedDict(TypedDict):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    customer: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""


class ListMandateResponseLinks(BaseModel):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: Annotated[URL, pydantic.Field(alias="self")]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    customer: URL
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""


class ListMandateResponseTypedDict(TypedDict):
    resource: str
    r"""Indicates the response contains a mandate object. Will always contain the string `mandate` for this endpoint."""
    id: str
    r"""The identifier uniquely referring to this mandate. Example: `mdt_pWUnw6pkBN`."""
    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""
    method: MandateMethodResponse
    r"""Payment method of the mandate.

    SEPA Direct Debit and PayPal mandates can be created directly.
    """
    details: ListMandateResponseDetailsTypedDict
    signature_date: Nullable[str]
    r"""The date when the mandate was signed in `YYYY-MM-DD` format."""
    mandate_reference: Nullable[str]
    r"""A custom mandate reference. For SEPA Direct Debit, it is vital to provide a unique reference. Some banks will
    decline Direct Debit payments if the mandate reference is not unique.
    """
    status: ListMandateResponseStatus
    customer_id: str
    r"""The identifier referring to the [customer](get-customer) this mandate was linked to."""
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""
    links: ListMandateResponseLinksTypedDict
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""


class ListMandateResponse(BaseModel):
    resource: str
    r"""Indicates the response contains a mandate object. Will always contain the string `mandate` for this endpoint."""

    id: str
    r"""The identifier uniquely referring to this mandate. Example: `mdt_pWUnw6pkBN`."""

    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""

    method: MandateMethodResponse
    r"""Payment method of the mandate.

    SEPA Direct Debit and PayPal mandates can be created directly.
    """

    details: ListMandateResponseDetails

    signature_date: Annotated[Nullable[str], pydantic.Field(alias="signatureDate")]
    r"""The date when the mandate was signed in `YYYY-MM-DD` format."""

    mandate_reference: Annotated[
        Nullable[str], pydantic.Field(alias="mandateReference")
    ]
    r"""A custom mandate reference. For SEPA Direct Debit, it is vital to provide a unique reference. Some banks will
    decline Direct Debit payments if the mandate reference is not unique.
    """

    status: ListMandateResponseStatus

    customer_id: Annotated[str, pydantic.Field(alias="customerId")]
    r"""The identifier referring to the [customer](get-customer) this mandate was linked to."""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""

    links: Annotated[ListMandateResponseLinks, pydantic.Field(alias="_links")]
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    @field_serializer("mode")
    def serialize_mode(self, value):
        if isinstance(value, str):
            try:
                return models.Mode(value)
            except ValueError:
                return value
        return value

    @field_serializer("method")
    def serialize_method(self, value):
        if isinstance(value, str):
            try:
                return models.MandateMethodResponse(value)
            except ValueError:
                return value
        return value

    @field_serializer("status")
    def serialize_status(self, value):
        if isinstance(value, str):
            try:
                return models.ListMandateResponseStatus(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


try:
    ListMandateResponseDetails.model_rebuild()
except NameError:
    pass
try:
    ListMandateResponseLinks.model_rebuild()
except NameError:
    pass
try:
    ListMandateResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 64e75db06c3f

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .amount_nullable import AmountNullable, AmountNullableTypedDict
from .balance_transaction_type import BalanceTransactionType
from mollie import models
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DeductionDetailsTypedDict(TypedDict):
    r"""A detailed breakdown of the deductions withheld from the movement. Each field represents a specific type of
    deduction applied to the transaction. Only the applicable fields will be present.
    """

    fees: NotRequired[Nullable[AmountNullableTypedDict]]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    commissions: NotRequired[Nullable[AmountNullableTypedDict]]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    repayments: NotRequired[Nullable[AmountNullableTypedDict]]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    reservations: NotRequired[Nullable[AmountNullableTypedDict]]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""


class DeductionDetails(BaseModel):
    r"""A detailed breakdown of the deductions withheld from the movement. Each field represents a specific type of
    deduction applied to the transaction. Only the applicable fields will be present.
    """

    fees: OptionalNullable[AmountNullable] = UNSET
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    commissions: OptionalNullable[AmountNullable] = UNSET
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    repayments: OptionalNullable[AmountNullable] = UNSET
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    reservations: OptionalNullable[AmountNullable] = UNSET
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["fees", "commissions", "repayments", "reservations"])
        nullable_fields = set(["fees", "commissions", "repayments", "reservations"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class EntityBalanceTransactionPaymentTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]


class EntityBalanceTransactionPayment(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CaptureTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    capture_id: NotRequired[str]
    capture_description: NotRequired[str]


class Capture(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    capture_id: Annotated[Optional[str], pydantic.Field(alias="captureId")] = None

    capture_description: Annotated[
        Optional[str], pydantic.Field(alias="captureDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "captureId", "captureDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CaptureCommisionTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    capture_id: NotRequired[str]
    capture_description: NotRequired[str]


class CaptureCommision(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    capture_id: Annotated[Optional[str], pydantic.Field(alias="captureId")] = None

    capture_description: Annotated[
        Optional[str], pydantic.Field(alias="captureDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "captureId", "captureDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CaptureRollingReserveReleaseTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    capture_id: NotRequired[str]
    capture_description: NotRequired[str]


class CaptureRollingReserveRelease(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    capture_id: Annotated[Optional[str], pydantic.Field(alias="captureId")] = None

    capture_description: Annotated[
        Optional[str], pydantic.Field(alias="captureDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "captureId", "captureDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UnauthorizedDirectDebitTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]


class UnauthorizedDirectDebit(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class FailedPaymentTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]


class FailedPayment(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class RefundTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    refund_id: NotRequired[str]
    refund_description: NotRequired[str]


class Refund(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    refund_id: Annotated[Optional[str], pydantic.Field(alias="refundId")] = None

    refund_description: Annotated[
        Optional[str], pydantic.Field(alias="refundDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "refundId", "refundDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class RefundCompensationTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    refund_id: NotRequired[str]
    refund_description: NotRequired[str]


class RefundCompensation(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    refund_id: Annotated[Optional[str], pydantic.Field(alias="refundId")] = None

    refund_description: Annotated[
        Optional[str], pydantic.Field(alias="refundDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "refundId", "refundDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReturnedRefundTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    refund_id: NotRequired[str]
    refund_description: NotRequired[str]


class ReturnedRefund(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    refund_id: Annotated[Optional[str], pydantic.Field(alias="refundId")] = None

    refund_description: Annotated[
        Optional[str], pydantic.Field(alias="refundDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "refundId", "refundDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReturnedRefundCompensationTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    refund_id: NotRequired[str]
    refund_description: NotRequired[str]


class ReturnedRefundCompensation(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    refund_id: Annotated[Optional[str], pydantic.Field(alias="refundId")] = None

    refund_description: Annotated[
        Optional[str], pydantic.Field(alias="refundDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "refundId", "refundDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ChargebackTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    chargeback_id: NotRequired[str]
    chargeback_description: NotRequired[str]


class Chargeback(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    chargeback_id: Annotated[Optional[str], pydantic.Field(alias="chargebackId")] = None

    chargeback_description: Annotated[
        Optional[str], pydantic.Field(alias="chargebackDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "chargebackId", "chargebackDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ChargebackReversalTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    chargeback_id: NotRequired[str]
    chargeback_description: NotRequired[str]


class ChargebackReversal(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    chargeback_id: Annotated[Optional[str], pydantic.Field(alias="chargebackId")] = None

    chargeback_description: Annotated[
        Optional[str], pydantic.Field(alias="chargebackDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "chargebackId", "chargebackDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ChargebackCompensationTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    chargeback_id: NotRequired[str]
    chargeback_description: NotRequired[str]


class ChargebackCompensation(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    chargeback_id: Annotated[Optional[str], pydantic.Field(alias="chargebackId")] = None

    chargeback_description: Annotated[
        Optional[str], pydantic.Field(alias="chargebackDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "chargebackId", "chargebackDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReversedChargebackCompensationTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    chargeback_id: NotRequired[str]
    chargeback_description: NotRequired[str]


class ReversedChargebackCompensation(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    chargeback_id: Annotated[Optional[str], pydantic.Field(alias="chargebackId")] = None

    chargeback_description: Annotated[
        Optional[str], pydantic.Field(alias="chargebackDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "chargebackId", "chargebackDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class OutgoingTransferTypedDict(TypedDict):
    transfer_id: NotRequired[str]
    settlement_id: NotRequired[str]


class OutgoingTransfer(BaseModel):
    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferId")] = None

    settlement_id: Annotated[Optional[str], pydantic.Field(alias="settlementId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["transferId", "settlementId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CanceledOutgoingTransferTypedDict(TypedDict):
    transfer_id: NotRequired[str]
    settlement_id: NotRequired[str]


class CanceledOutgoingTransfer(BaseModel):
    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferId")] = None

    settlement_id: Annotated[Optional[str], pydantic.Field(alias="settlementId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["transferId", "settlementId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReturnedTransferTypedDict(TypedDict):
    transfer_id: NotRequired[str]
    settlement_id: NotRequired[str]


class ReturnedTransfer(BaseModel):
    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferId")] = None

    settlement_id: Annotated[Optional[str], pydantic.Field(alias="settlementId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["transferId", "settlementId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class InvoiceCompensationTypedDict(TypedDict):
    invoice_id: NotRequired[str]


class InvoiceCompensation(BaseModel):
    invoice_id: Annotated[Optional[str], pydantic.Field(alias="invoiceId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["invoiceId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class EntityBalanceTransactionApplicationFeeTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    paying_owner: NotRequired[str]


class EntityBalanceTransactionApplicationFee(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    paying_owner: Annotated[Optional[str], pydantic.Field(alias="payingOwner")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription", "payingOwner"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class SplitPaymentTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    payment_onwer: NotRequired[str]


class SplitPayment(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    payment_onwer: Annotated[Optional[str], pydantic.Field(alias="paymentOnwer")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription", "paymentOnwer"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PlatformPaymentRefundTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    refund_id: NotRequired[str]
    refund_description: NotRequired[str]


class PlatformPaymentRefund(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    refund_id: Annotated[Optional[str], pydantic.Field(alias="refundId")] = None

    refund_description: Annotated[
        Optional[str], pydantic.Field(alias="refundDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "refundId", "refundDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReturnedPlatformPaymentRefundTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    refund_id: NotRequired[str]
    refund_description: NotRequired[str]


class ReturnedPlatformPaymentRefund(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    refund_id: Annotated[Optional[str], pydantic.Field(alias="refundId")] = None

    refund_description: Annotated[
        Optional[str], pydantic.Field(alias="refundDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "refundId", "refundDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PlatformPaymentChargebackTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    chargeback_id: NotRequired[str]
    chargeback_description: NotRequired[str]


class PlatformPaymentChargeback(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    chargeback_id: Annotated[Optional[str], pydantic.Field(alias="chargebackId")] = None

    chargeback_description: Annotated[
        Optional[str], pydantic.Field(alias="chargebackDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "chargebackId", "chargebackDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReversedPlatformPaymentChargebackTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    chargeback_id: NotRequired[str]
    chargeback_description: NotRequired[str]


class ReversedPlatformPaymentChargeback(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    chargeback_id: Annotated[Optional[str], pydantic.Field(alias="chargebackId")] = None

    chargeback_description: Annotated[
        Optional[str], pydantic.Field(alias="chargebackDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["paymentId", "paymentDescription", "chargebackId", "chargebackDescription"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PaymentCommissionTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]
    organization_id: NotRequired[str]


class PaymentCommission(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    organization_id: Annotated[
        Optional[str], pydantic.Field(alias="organizationId")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription", "organizationId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReimbursementFeeTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]


class ReimbursementFee(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class FailedPaymentFeeTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]


class FailedPaymentFee(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PaymentFeeTypedDict(TypedDict):
    payment_id: NotRequired[str]
    payment_description: NotRequired[str]


class PaymentFee(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    payment_description: Annotated[
        Optional[str], pydantic.Field(alias="paymentDescription")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId", "paymentDescription"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ManagedFeeTypedDict(TypedDict):
    fee_type: NotRequired[str]
    fee_id: NotRequired[str]


class ManagedFee(BaseModel):
    fee_type: Annotated[Optional[str], pydantic.Field(alias="feeType")] = None

    fee_id: Annotated[Optional[str], pydantic.Field(alias="feeId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["feeType", "feeId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ReturnedManagedFeeTypedDict(TypedDict):
    fee_type: NotRequired[str]
    fee_id: NotRequired[str]


class ReturnedManagedFee(BaseModel):
    fee_type: Annotated[Optional[str], pydantic.Field(alias="feeType")] = None

    fee_id: Annotated[Optional[str], pydantic.Field(alias="feeId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["feeType", "feeId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PostPaymentSplitPaymentTypedDict(TypedDict):
    payment_id: NotRequired[str]


class PostPaymentSplitPayment(BaseModel):
    payment_id: Annotated[Optional[str], pydantic.Field(alias="paymentId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["paymentId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ContextTypedDict(TypedDict):
    r"""Depending on the type of the balance transaction, we will try to give more context about the specific event that
    triggered it. For example, the context object for a payment transaction will look like
    `{\"paymentId\": \"tr_5B8cwPMGnU6qLbRvo7qEZo\", \"paymentDescription\": \"Description\"}`.

    Below is a complete list of the context values that each type of transaction will have.

    * Type `payment`: `paymentId`, `paymentDescription`
    * Type `capture`: `paymentId` `captureId`, `paymentDescription`, `captureDescription`
    * Type `capture-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `capture-rolling-reserve-release`: `paymentId`, `paymentDescription`, `captureId`, `captureDescription`
    * Type `unauthorized-direct-debit`: `paymentId`, `paymentDescription`
    * Type `failed-payment`: `paymentId`, `paymentDescription`
    * Type `refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-reversal`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `outgoing-transfer`: `settlementId` `transferId`
    * Type `canceled-outgoing-transfer`: `settlementId` `transferId`
    * Type `returned-transfer`: `settlementId` `transferId`
    * Type `invoice-compensation`: `invoiceId`
    * Type `balance-correction`: none
    * Type `application-fee`: `paymentId`, `paymentDescription`, `payingOwner`
    * Type `split-payment`: `paymentId`, `paymentDescription`, `paymentOwner`
    * Type `platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `payment-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `reimbursement-fee`: `paymentId`, `paymentDescription`
    * Type `failed-payment-fee`: `paymentId`, `paymentDescription`
    * Type `payment-fee`: `paymentId`, `paymentDescription`
    * Type `cash-advance-loan`: none
    * Type `platform-connected-organizations-fee`: none
    * Type `managed-fee`: `feeType`, `<name of the fee>Id`
    * Type `returned-managed-fee`:  `feeType`, `<name of the fee>Id`
    * Type `topup`: none
    * Type `balance-reserve`: none
    * Type `balance-reserve-return`: none
    * Type `movement`: none
    * Type `post-payment-split-payment`: `paymentId`
    * Type `cash-collateral-issuance`: none
    * Type `cash-collateral-release`: none
    """

    payment: NotRequired[Nullable[EntityBalanceTransactionPaymentTypedDict]]
    capture: NotRequired[Nullable[CaptureTypedDict]]
    capture_commision: NotRequired[Nullable[CaptureCommisionTypedDict]]
    capture_rolling_reserve_release: NotRequired[
        Nullable[CaptureRollingReserveReleaseTypedDict]
    ]
    unauthorized_direct_debit: NotRequired[Nullable[UnauthorizedDirectDebitTypedDict]]
    failed_payment: NotRequired[Nullable[FailedPaymentTypedDict]]
    refund: NotRequired[Nullable[RefundTypedDict]]
    refund_compensation: NotRequired[Nullable[RefundCompensationTypedDict]]
    returned_refund: NotRequired[Nullable[ReturnedRefundTypedDict]]
    returned_refund_compensation: NotRequired[
        Nullable[ReturnedRefundCompensationTypedDict]
    ]
    chargeback: NotRequired[Nullable[ChargebackTypedDict]]
    chargeback_reversal: NotRequired[Nullable[ChargebackReversalTypedDict]]
    chargeback_compensation: NotRequired[Nullable[ChargebackCompensationTypedDict]]
    reversed_chargeback_compensation: NotRequired[
        Nullable[ReversedChargebackCompensationTypedDict]
    ]
    outgoing_transfer: NotRequired[Nullable[OutgoingTransferTypedDict]]
    canceled_outgoing_transfer: NotRequired[Nullable[CanceledOutgoingTransferTypedDict]]
    returned_transfer: NotRequired[Nullable[ReturnedTransferTypedDict]]
    invoice_compensation: NotRequired[Nullable[InvoiceCompensationTypedDict]]
    application_fee: NotRequired[
        Nullable[EntityBalanceTransactionApplicationFeeTypedDict]
    ]
    split_payment: NotRequired[Nullable[SplitPaymentTypedDict]]
    platform_payment_refund: NotRequired[Nullable[PlatformPaymentRefundTypedDict]]
    returned_platform_payment_refund: NotRequired[
        Nullable[ReturnedPlatformPaymentRefundTypedDict]
    ]
    platform_payment_chargeback: NotRequired[
        Nullable[PlatformPaymentChargebackTypedDict]
    ]
    reversed_platform_payment_chargeback: NotRequired[
        Nullable[ReversedPlatformPaymentChargebackTypedDict]
    ]
    payment_commission: NotRequired[Nullable[PaymentCommissionTypedDict]]
    reimbursement_fee: NotRequired[Nullable[ReimbursementFeeTypedDict]]
    failed_payment_fee: NotRequired[Nullable[FailedPaymentFeeTypedDict]]
    payment_fee: NotRequired[Nullable[PaymentFeeTypedDict]]
    managed_fee: NotRequired[Nullable[ManagedFeeTypedDict]]
    returned_managed_fee: NotRequired[Nullable[ReturnedManagedFeeTypedDict]]
    post_payment_split_payment: NotRequired[Nullable[PostPaymentSplitPaymentTypedDict]]


class Context(BaseModel):
    r"""Depending on the type of the balance transaction, we will try to give more context about the specific event that
    triggered it. For example, the context object for a payment transaction will look like
    `{\"paymentId\": \"tr_5B8cwPMGnU6qLbRvo7qEZo\", \"paymentDescription\": \"Description\"}`.

    Below is a complete list of the context values that each type of transaction will have.

    * Type `payment`: `paymentId`, `paymentDescription`
    * Type `capture`: `paymentId` `captureId`, `paymentDescription`, `captureDescription`
    * Type `capture-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `capture-rolling-reserve-release`: `paymentId`, `paymentDescription`, `captureId`, `captureDescription`
    * Type `unauthorized-direct-debit`: `paymentId`, `paymentDescription`
    * Type `failed-payment`: `paymentId`, `paymentDescription`
    * Type `refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-reversal`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `outgoing-transfer`: `settlementId` `transferId`
    * Type `canceled-outgoing-transfer`: `settlementId` `transferId`
    * Type `returned-transfer`: `settlementId` `transferId`
    * Type `invoice-compensation`: `invoiceId`
    * Type `balance-correction`: none
    * Type `application-fee`: `paymentId`, `paymentDescription`, `payingOwner`
    * Type `split-payment`: `paymentId`, `paymentDescription`, `paymentOwner`
    * Type `platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `payment-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `reimbursement-fee`: `paymentId`, `paymentDescription`
    * Type `failed-payment-fee`: `paymentId`, `paymentDescription`
    * Type `payment-fee`: `paymentId`, `paymentDescription`
    * Type `cash-advance-loan`: none
    * Type `platform-connected-organizations-fee`: none
    * Type `managed-fee`: `feeType`, `<name of the fee>Id`
    * Type `returned-managed-fee`:  `feeType`, `<name of the fee>Id`
    * Type `topup`: none
    * Type `balance-reserve`: none
    * Type `balance-reserve-return`: none
    * Type `movement`: none
    * Type `post-payment-split-payment`: `paymentId`
    * Type `cash-collateral-issuance`: none
    * Type `cash-collateral-release`: none
    """

    payment: OptionalNullable[EntityBalanceTransactionPayment] = UNSET

    capture: OptionalNullable[Capture] = UNSET

    capture_commision: Annotated[
        OptionalNullable[CaptureCommision], pydantic.Field(alias="capture-commision")
    ] = UNSET

    capture_rolling_reserve_release: Annotated[
        OptionalNullable[CaptureRollingReserveRelease],
        pydantic.Field(alias="capture-rolling-reserve-release"),
    ] = UNSET

    unauthorized_direct_debit: Annotated[
        OptionalNullable[UnauthorizedDirectDebit],
        pydantic.Field(alias="unauthorized-direct-debit"),
    ] = UNSET

    failed_payment: Annotated[
        OptionalNullable[FailedPayment], pydantic.Field(alias="failed-payment")
    ] = UNSET

    refund: OptionalNullable[Refund] = UNSET

    refund_compensation: Annotated[
        OptionalNullable[RefundCompensation],
        pydantic.Field(alias="refund-compensation"),
    ] = UNSET

    returned_refund: Annotated[
        OptionalNullable[ReturnedRefund], pydantic.Field(alias="returned-refund")
    ] = UNSET

    returned_refund_compensation: Annotated[
        OptionalNullable[ReturnedRefundCompensation],
        pydantic.Field(alias="returned-refund-compensation"),
    ] = UNSET

    chargeback: OptionalNullable[Chargeback] = UNSET

    chargeback_reversal: Annotated[
        OptionalNullable[ChargebackReversal],
        pydantic.Field(alias="chargeback-reversal"),
    ] = UNSET

    chargeback_compensation: Annotated[
        OptionalNullable[ChargebackCompensation],
        pydantic.Field(alias="chargeback-compensation"),
    ] = UNSET

    reversed_chargeback_compensation: Annotated[
        OptionalNullable[ReversedChargebackCompensation],
        pydantic.Field(alias="reversed-chargeback-compensation"),
    ] = UNSET

    outgoing_transfer: Annotated[
        OptionalNullable[OutgoingTransfer], pydantic.Field(alias="outgoing-transfer")
    ] = UNSET

    canceled_outgoing_transfer: Annotated[
        OptionalNullable[CanceledOutgoingTransfer],
        pydantic.Field(alias="canceled-outgoing-transfer"),
    ] = UNSET

    returned_transfer: Annotated[
        OptionalNullable[ReturnedTransfer], pydantic.Field(alias="returned-transfer")
    ] = UNSET

    invoice_compensation: Annotated[
        OptionalNullable[InvoiceCompensation],
        pydantic.Field(alias="invoice-compensation"),
    ] = UNSET

    application_fee: Annotated[
        OptionalNullable[EntityBalanceTransactionApplicationFee],
        pydantic.Field(alias="application-fee"),
    ] = UNSET

    split_payment: Annotated[
        OptionalNullable[SplitPayment], pydantic.Field(alias="split-payment")
    ] = UNSET

    platform_payment_refund: Annotated[
        OptionalNullable[PlatformPaymentRefund],
        pydantic.Field(alias="platform-payment-refund"),
    ] = UNSET

    returned_platform_payment_refund: Annotated[
        OptionalNullable[ReturnedPlatformPaymentRefund],
        pydantic.Field(alias="returned-platform-payment-refund"),
    ] = UNSET

    platform_payment_chargeback: Annotated[
        OptionalNullable[PlatformPaymentChargeback],
        pydantic.Field(alias="platform-payment-chargeback"),
    ] = UNSET

    reversed_platform_payment_chargeback: Annotated[
        OptionalNullable[ReversedPlatformPaymentChargeback],
        pydantic.Field(alias="reversed-platform-payment-chargeback"),
    ] = UNSET

    payment_commission: Annotated[
        OptionalNullable[PaymentCommission], pydantic.Field(alias="payment-commission")
    ] = UNSET

    reimbursement_fee: Annotated[
        OptionalNullable[ReimbursementFee], pydantic.Field(alias="reimbursement-fee")
    ] = UNSET

    failed_payment_fee: Annotated[
        OptionalNullable[FailedPaymentFee], pydantic.Field(alias="failed-payment-fee")
    ] = UNSET

    payment_fee: Annotated[
        OptionalNullable[PaymentFee], pydantic.Field(alias="payment-fee")
    ] = UNSET

    managed_fee: Annotated[
        OptionalNullable[ManagedFee], pydantic.Field(alias="managed-fee")
    ] = UNSET

    returned_managed_fee: Annotated[
        OptionalNullable[ReturnedManagedFee],
        pydantic.Field(alias="returned-managed-fee"),
    ] = UNSET

    post_payment_split_payment: Annotated[
        OptionalNullable[PostPaymentSplitPayment],
        pydantic.Field(alias="post-payment-split-payment"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "payment",
                "capture",
                "capture-commision",
                "capture-rolling-reserve-release",
                "unauthorized-direct-debit",
                "failed-payment",
                "refund",
                "refund-compensation",
                "returned-refund",
                "returned-refund-compensation",
                "chargeback",
                "chargeback-reversal",
                "chargeback-compensation",
                "reversed-chargeback-compensation",
                "outgoing-transfer",
                "canceled-outgoing-transfer",
                "returned-transfer",
                "invoice-compensation",
                "application-fee",
                "split-payment",
                "platform-payment-refund",
                "returned-platform-payment-refund",
                "platform-payment-chargeback",
                "reversed-platform-payment-chargeback",
                "payment-commission",
                "reimbursement-fee",
                "failed-payment-fee",
                "payment-fee",
                "managed-fee",
                "returned-managed-fee",
                "post-payment-split-payment",
            ]
        )
        nullable_fields = set(
            [
                "payment",
                "capture",
                "capture-commision",
                "capture-rolling-reserve-release",
                "unauthorized-direct-debit",
                "failed-payment",
                "refund",
                "refund-compensation",
                "returned-refund",
                "returned-refund-compensation",
                "chargeback",
                "chargeback-reversal",
                "chargeback-compensation",
                "reversed-chargeback-compensation",
                "outgoing-transfer",
                "canceled-outgoing-transfer",
                "returned-transfer",
                "invoice-compensation",
                "application-fee",
                "split-payment",
                "platform-payment-refund",
                "returned-platform-payment-refund",
                "platform-payment-chargeback",
                "reversed-platform-payment-chargeback",
                "payment-commission",
                "reimbursement-fee",
                "failed-payment-fee",
                "payment-fee",
                "managed-fee",
                "returned-managed-fee",
                "post-payment-split-payment",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class EntityBalanceTransactionTypedDict(TypedDict):
    resource: str
    r"""Indicates the response contains a balance transaction object. Will always contain the string `balance-transaction`
    for this endpoint.
    """
    id: str
    r"""The identifier uniquely referring to this balance transaction."""
    type: BalanceTransactionType
    result_amount: AmountTypedDict
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    initial_amount: AmountTypedDict
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""
    deductions: NotRequired[Nullable[AmountNullableTypedDict]]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    deduction_details: NotRequired[Nullable[DeductionDetailsTypedDict]]
    r"""A detailed breakdown of the deductions withheld from the movement. Each field represents a specific type of
    deduction applied to the transaction. Only the applicable fields will be present.
    """
    context: NotRequired[Nullable[ContextTypedDict]]
    r"""Depending on the type of the balance transaction, we will try to give more context about the specific event that
    triggered it. For example, the context object for a payment transaction will look like
    `{\"paymentId\": \"tr_5B8cwPMGnU6qLbRvo7qEZo\", \"paymentDescription\": \"Description\"}`.

    Below is a complete list of the context values that each type of transaction will have.

    * Type `payment`: `paymentId`, `paymentDescription`
    * Type `capture`: `paymentId` `captureId`, `paymentDescription`, `captureDescription`
    * Type `capture-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `capture-rolling-reserve-release`: `paymentId`, `paymentDescription`, `captureId`, `captureDescription`
    * Type `unauthorized-direct-debit`: `paymentId`, `paymentDescription`
    * Type `failed-payment`: `paymentId`, `paymentDescription`
    * Type `refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-reversal`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `outgoing-transfer`: `settlementId` `transferId`
    * Type `canceled-outgoing-transfer`: `settlementId` `transferId`
    * Type `returned-transfer`: `settlementId` `transferId`
    * Type `invoice-compensation`: `invoiceId`
    * Type `balance-correction`: none
    * Type `application-fee`: `paymentId`, `paymentDescription`, `payingOwner`
    * Type `split-payment`: `paymentId`, `paymentDescription`, `paymentOwner`
    * Type `platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `payment-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `reimbursement-fee`: `paymentId`, `paymentDescription`
    * Type `failed-payment-fee`: `paymentId`, `paymentDescription`
    * Type `payment-fee`: `paymentId`, `paymentDescription`
    * Type `cash-advance-loan`: none
    * Type `platform-connected-organizations-fee`: none
    * Type `managed-fee`: `feeType`, `<name of the fee>Id`
    * Type `returned-managed-fee`:  `feeType`, `<name of the fee>Id`
    * Type `topup`: none
    * Type `balance-reserve`: none
    * Type `balance-reserve-return`: none
    * Type `movement`: none
    * Type `post-payment-split-payment`: `paymentId`
    * Type `cash-collateral-issuance`: none
    * Type `cash-collateral-release`: none
    """


class EntityBalanceTransaction(BaseModel):
    resource: str
    r"""Indicates the response contains a balance transaction object. Will always contain the string `balance-transaction`
    for this endpoint.
    """

    id: str
    r"""The identifier uniquely referring to this balance transaction."""

    type: BalanceTransactionType

    result_amount: Annotated[Amount, pydantic.Field(alias="resultAmount")]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    initial_amount: Annotated[Amount, pydantic.Field(alias="initialAmount")]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""

    deductions: OptionalNullable[AmountNullable] = UNSET
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    deduction_details: Annotated[
        OptionalNullable[DeductionDetails], pydantic.Field(alias="deductionDetails")
    ] = UNSET
    r"""A detailed breakdown of the deductions withheld from the movement. Each field represents a specific type of
    deduction applied to the transaction. Only the applicable fields will be present.
    """

    context: OptionalNullable[Context] = UNSET
    r"""Depending on the type of the balance transaction, we will try to give more context about the specific event that
    triggered it. For example, the context object for a payment transaction will look like
    `{\"paymentId\": \"tr_5B8cwPMGnU6qLbRvo7qEZo\", \"paymentDescription\": \"Description\"}`.

    Below is a complete list of the context values that each type of transaction will have.

    * Type `payment`: `paymentId`, `paymentDescription`
    * Type `capture`: `paymentId` `captureId`, `paymentDescription`, `captureDescription`
    * Type `capture-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `capture-rolling-reserve-release`: `paymentId`, `paymentDescription`, `captureId`, `captureDescription`
    * Type `unauthorized-direct-debit`: `paymentId`, `paymentDescription`
    * Type `failed-payment`: `paymentId`, `paymentDescription`
    * Type `refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-refund-compensation`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-reversal`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-chargeback-compensation`: `paymentId`, `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `outgoing-transfer`: `settlementId` `transferId`
    * Type `canceled-outgoing-transfer`: `settlementId` `transferId`
    * Type `returned-transfer`: `settlementId` `transferId`
    * Type `invoice-compensation`: `invoiceId`
    * Type `balance-correction`: none
    * Type `application-fee`: `paymentId`, `paymentDescription`, `payingOwner`
    * Type `split-payment`: `paymentId`, `paymentDescription`, `paymentOwner`
    * Type `platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `returned-platform-payment-refund`: `paymentId` `refundId`, `paymentDescription`, `refundDescription`
    * Type `platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `reversed-platform-payment-chargeback`: `paymentId` `chargebackId`, `paymentDescription`, `chargebackDescription`
    * Type `payment-commission`: `paymentId`, `paymentDescription`, `organizationId`
    * Type `reimbursement-fee`: `paymentId`, `paymentDescription`
    * Type `failed-payment-fee`: `paymentId`, `paymentDescription`
    * Type `payment-fee`: `paymentId`, `paymentDescription`
    * Type `cash-advance-loan`: none
    * Type `platform-connected-organizations-fee`: none
    * Type `managed-fee`: `feeType`, `<name of the fee>Id`
    * Type `returned-managed-fee`:  `feeType`, `<name of the fee>Id`
    * Type `topup`: none
    * Type `balance-reserve`: none
    * Type `balance-reserve-return`: none
    * Type `movement`: none
    * Type `post-payment-split-payment`: `paymentId`
    * Type `cash-collateral-issuance`: none
    * Type `cash-collateral-release`: none
    """

    @field_serializer("type")
    def serialize_type(self, value):
        if isinstance(value, str):
            try:
                return models.BalanceTransactionType(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["deductions", "deductionDetails", "context"])
        nullable_fields = set(["deductions", "deductionDetails", "context"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    EntityBalanceTransactionPayment.model_rebuild()
except NameError:
    pass
try:
    Capture.model_rebuild()
except NameError:
    pass
try:
    CaptureCommision.model_rebuild()
except NameError:
    pass
try:
    CaptureRollingReserveRelease.model_rebuild()
except NameError:
    pass
try:
    UnauthorizedDirectDebit.model_rebuild()
except NameError:
    pass
try:
    FailedPayment.model_rebuild()
except NameError:
    pass
try:
    Refund.model_rebuild()
except NameError:
    pass
try:
    RefundCompensation.model_rebuild()
except NameError:
    pass
try:
    ReturnedRefund.model_rebuild()
except NameError:
    pass
try:
    ReturnedRefundCompensation.model_rebuild()
except NameError:
    pass
try:
    Chargeback.model_rebuild()
except NameError:
    pass
try:
    ChargebackReversal.model_rebuild()
except NameError:
    pass
try:
    ChargebackCompensation.model_rebuild()
except NameError:
    pass
try:
    ReversedChargebackCompensation.model_rebuild()
except NameError:
    pass
try:
    OutgoingTransfer.model_rebuild()
except NameError:
    pass
try:
    CanceledOutgoingTransfer.model_rebuild()
except NameError:
    pass
try:
    ReturnedTransfer.model_rebuild()
except NameError:
    pass
try:
    InvoiceCompensation.model_rebuild()
except NameError:
    pass
try:
    EntityBalanceTransactionApplicationFee.model_rebuild()
except NameError:
    pass
try:
    SplitPayment.model_rebuild()
except NameError:
    pass
try:
    PlatformPaymentRefund.model_rebuild()
except NameError:
    pass
try:
    ReturnedPlatformPaymentRefund.model_rebuild()
except NameError:
    pass
try:
    PlatformPaymentChargeback.model_rebuild()
except NameError:
    pass
try:
    ReversedPlatformPaymentChargeback.model_rebuild()
except NameError:
    pass
try:
    PaymentCommission.model_rebuild()
except NameError:
    pass
try:
    ReimbursementFee.model_rebuild()
except NameError:
    pass
try:
    FailedPaymentFee.model_rebuild()
except NameError:
    pass
try:
    PaymentFee.model_rebuild()
except NameError:
    pass
try:
    ManagedFee.model_rebuild()
except NameError:
    pass
try:
    ReturnedManagedFee.model_rebuild()
except NameError:
    pass
try:
    PostPaymentSplitPayment.model_rebuild()
except NameError:
    pass
try:
    Context.model_rebuild()
except NameError:
    pass
try:
    EntityBalanceTransaction.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 30281f1be744

from __future__ import annotations
from .url import URL, URLTypedDict
from .url_nullable import URLNullable, URLNullableTypedDict
from mollie.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, TypedDict


class ListLinksTypedDict(TypedDict):
    r"""Links to help navigate through the lists of items. Every URL object will contain an `href` and a `type` field."""

    self_: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    previous: Nullable[URLNullableTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    next: Nullable[URLNullableTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    documentation: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""


class ListLinks(BaseModel):
    r"""Links to help navigate through the lists of items. Every URL object will contain an `href` and a `type` field."""

    self_: Annotated[URL, pydantic.Field(alias="self")]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    previous: Nullable[URLNullable]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    next: Nullable[URLNullable]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    documentation: URL
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


try:
    ListLinks.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 001af0875f6e

from __future__ import annotations
from enum import Enum


class RefundExternalReferenceType(str, Enum):
    r"""Specifies the reference type"""

    ACQUIRER_REFERENCE = "acquirer-reference"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2e7e96ce3758

from __future__ import annotations
from .url import URL, URLTypedDict
from enum import Enum
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from mollie.utils import FieldMetadata, HeaderMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetPartnerStatusRequestTypedDict(TypedDict):
    idempotency_key: NotRequired[str]
    r"""A unique key to ensure idempotent requests. This key should be a UUID v4 string."""


class GetPartnerStatusRequest(BaseModel):
    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A unique key to ensure idempotent requests. This key should be a UUID v4 string."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["idempotency-key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class PartnerType(str, Enum):
    r"""Indicates the type of partner. Will be `null` if the currently authenticated organization is not
    enrolled as a partner.
    """

    OAUTH = "oauth"
    SIGNUPLINK = "signuplink"
    USERAGENT = "useragent"


class UserAgentTokenTypedDict(TypedDict):
    token: NotRequired[str]
    r"""The unique User-Agent token."""
    starts_at: NotRequired[str]
    r"""The date from which the token is active, in ISO 8601 format."""
    ends_at: NotRequired[Nullable[str]]
    r"""The date until when the token will be active, in ISO 8601 format. Will be `null` if the token
    does not have an end date (yet).
    """


class UserAgentToken(BaseModel):
    token: Optional[str] = None
    r"""The unique User-Agent token."""

    starts_at: Annotated[Optional[str], pydantic.Field(alias="startsAt")] = None
    r"""The date from which the token is active, in ISO 8601 format."""

    ends_at: Annotated[OptionalNullable[str], pydantic.Field(alias="endsAt")] = UNSET
    r"""The date until when the token will be active, in ISO 8601 format. Will be `null` if the token
    does not have an end date (yet).
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["token", "startsAt", "endsAt"])
        nullable_fields = set(["endsAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class GetPartnerStatusLinksTypedDict(TypedDict):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    signuplink: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    documentation: NotRequired[URLTypedDict]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""


class GetPartnerStatusLinks(BaseModel):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: Annotated[Optional[URL], pydantic.Field(alias="self")] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    signuplink: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    documentation: Optional[URL] = None
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["self", "signuplink", "documentation"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class GetPartnerStatusResponseTypedDict(TypedDict):
    r"""The partner status object."""

    resource: str
    r"""Indicates the response contains a partner status object. Will always contain the string `partner` for
    this endpoint.
    """
    partner_type: Nullable[PartnerType]
    r"""Indicates the type of partner. Will be `null` if the currently authenticated organization is not
    enrolled as a partner.
    """
    is_commission_partner: NotRequired[bool]
    r"""Whether the current organization is receiving commissions."""
    user_agent_tokens: NotRequired[List[UserAgentTokenTypedDict]]
    r"""Array of User-Agent token objects. Present if the organization is a partner of type `useragent`, or if
    they were in the past.
    """
    partner_contract_signed_at: NotRequired[Nullable[str]]
    r"""The date the partner contract was signed, in ISO 8601 format. Omitted if no contract has been signed
    (yet).
    """
    partner_contract_update_available: NotRequired[bool]
    r"""Whether an update to the partner contract is available and requiring the organization's agreement."""
    partner_contract_expires_at: NotRequired[str]
    r"""The expiration date of the signed partner contract, in ISO 8601 format. Omitted if contract has no
    expiration date (yet).
    """
    links: NotRequired[GetPartnerStatusLinksTypedDict]
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""


class GetPartnerStatusResponse(BaseModel):
    r"""The partner status object."""

    resource: str
    r"""Indicates the response contains a partner status object. Will always contain the string `partner` for
    this endpoint.
    """

    partner_type: Annotated[Nullable[PartnerType], pydantic.Field(alias="partnerType")]
    r"""Indicates the type of partner. Will be `null` if the currently authenticated organization is not
    enrolled as a partner.
    """

    is_commission_partner: Annotated[
        Optional[bool], pydantic.Field(alias="isCommissionPartner")
    ] = None
    r"""Whether the current organization is receiving commissions."""

    user_agent_tokens: Annotated[
        Optional[List[UserAgentToken]], pydantic.Field(alias="userAgentTokens")
    ] = None
    r"""Array of User-Agent token objects. Present if the organization is a partner of type `useragent`, or if
    they were in the past.
    """

    partner_contract_signed_at: Annotated[
        OptionalNullable[str], pydantic.Field(alias="partnerContractSignedAt")
    ] = UNSET
    r"""The date the partner contract was signed, in ISO 8601 format. Omitted if no contract has been signed
    (yet).
    """

    partner_contract_update_available: Annotated[
        Optional[bool], pydantic.Field(alias="partnerContractUpdateAvailable")
    ] = None
    r"""Whether an update to the partner contract is available and requiring the organization's agreement."""

    partner_contract_expires_at: Annotated[
        Optional[str], pydantic.Field(alias="partnerContractExpiresAt")
    ] = None
    r"""The expiration date of the signed partner contract, in ISO 8601 format. Omitted if contract has no
    expiration date (yet).
    """

    links: Annotated[
        Optional[GetPartnerStatusLinks], pydantic.Field(alias="_links")
    ] = None
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "isCommissionPartner",
                "userAgentTokens",
                "partnerContractSignedAt",
                "partnerContractUpdateAvailable",
                "partnerContractExpiresAt",
                "_links",
            ]
        )
        nullable_fields = set(["partnerType", "partnerContractSignedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    UserAgentToken.model_rebuild()
except NameError:
    pass
try:
    GetPartnerStatusLinks.model_rebuild()
except NameError:
    pass
try:
    GetPartnerStatusResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 830db198aae3

from __future__ import annotations
from .amount_nullable import AmountNullable, AmountNullableTypedDict
from .metadata import Metadata, MetadataTypedDict
from .mode import Mode
from .url import URL, URLTypedDict
from .url_nullable import URLNullable, URLNullableTypedDict
from enum import Enum
from mollie import models, utils
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CaptureResponseSettlementAmountTypedDict(TypedDict):
    r"""This optional field will contain the approximate amount that will be settled to your account, converted to the
    currency your account is settled in.

    Since the field contains an estimated amount during capture processing, it may change over time. To retrieve
    accurate settlement amounts we recommend using the [List balance transactions endpoint](list-balance-transactions)
    instead.
    """

    currency: str
    r"""A three-character ISO 4217 currency code."""
    value: str
    r"""A string containing an exact monetary amount in the given currency."""


class CaptureResponseSettlementAmount(BaseModel):
    r"""This optional field will contain the approximate amount that will be settled to your account, converted to the
    currency your account is settled in.

    Since the field contains an estimated amount during capture processing, it may change over time. To retrieve
    accurate settlement amounts we recommend using the [List balance transactions endpoint](list-balance-transactions)
    instead.
    """

    currency: str
    r"""A three-character ISO 4217 currency code."""

    value: str
    r"""A string containing an exact monetary amount in the given currency."""


class CaptureResponseStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The capture's status."""

    PENDING = "pending"
    SUCCEEDED = "succeeded"
    FAILED = "failed"


class CaptureResponseLinksTypedDict(TypedDict):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    payment: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    documentation: URLTypedDict
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    settlement: NotRequired[Nullable[URLNullableTypedDict]]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""
    shipment: NotRequired[Nullable[URLNullableTypedDict]]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""


class CaptureResponseLinks(BaseModel):
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    self_: Annotated[URL, pydantic.Field(alias="self")]
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    payment: URL
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    documentation: URL
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    settlement: OptionalNullable[URLNullable] = UNSET
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    shipment: OptionalNullable[URLNullable] = UNSET
    r"""In v2 endpoints, URLs are commonly represented as objects with an `href` and `type` field."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["settlement", "shipment"])
        nullable_fields = set(["settlement", "shipment"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class CaptureResponseTypedDict(TypedDict):
    resource: str
    r"""Indicates the response contains a capture object. Will always contain the string `capture` for this endpoint."""
    id: str
    r"""The identifier uniquely referring to this capture. Example: `cpt_mNepDkEtco6ah3QNPUGYH`."""
    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""
    amount: Nullable[AmountNullableTypedDict]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    status: CaptureResponseStatus
    payment_id: str
    r"""The unique identifier of the payment this capture was created for. For example: `tr_5B8cwPMGnU6qLbRvo7qEZo`.
    The full payment object can be retrieved via the payment URL in the `_links` object.
    """
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""
    links: CaptureResponseLinksTypedDict
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""
    description: NotRequired[str]
    r"""The description of the capture."""
    settlement_amount: NotRequired[Nullable[CaptureResponseSettlementAmountTypedDict]]
    r"""This optional field will contain the approximate amount that will be settled to your account, converted to the
    currency your account is settled in.

    Since the field contains an estimated amount during capture processing, it may change over time. To retrieve
    accurate settlement amounts we recommend using the [List balance transactions endpoint](list-balance-transactions)
    instead.
    """
    metadata: NotRequired[Nullable[MetadataTypedDict]]
    r"""Provide any data you like, for example a string or a JSON object. We will save the data alongside the entity. Whenever
    you fetch the entity with our API, we will also include the metadata. You can use up to approximately 1kB.
    """
    shipment_id: NotRequired[Nullable[str]]
    r"""The unique identifier of the shipment that triggered the creation of this capture, if applicable. For example:
    `shp_gNapNy9qQTUFZYnCrCF7J`.
    """
    settlement_id: NotRequired[Nullable[str]]
    r"""The identifier referring to the settlement this capture was settled with. For example, `stl_BkEjN2eBb`. This field
    is omitted if the capture is not settled (yet).
    """


class CaptureResponse(BaseModel):
    resource: str
    r"""Indicates the response contains a capture object. Will always contain the string `capture` for this endpoint."""

    id: str
    r"""The identifier uniquely referring to this capture. Example: `cpt_mNepDkEtco6ah3QNPUGYH`."""

    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""

    amount: Nullable[AmountNullable]
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    status: CaptureResponseStatus

    payment_id: Annotated[str, pydantic.Field(alias="paymentId")]
    r"""The unique identifier of the payment this capture was created for. For example: `tr_5B8cwPMGnU6qLbRvo7qEZo`.
    The full payment object can be retrieved via the payment URL in the `_links` object.
    """

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""

    links: Annotated[CaptureResponseLinks, pydantic.Field(alias="_links")]
    r"""An object with several relevant URLs. Every URL object will contain an `href` and a `type` field."""

    description: Optional[str] = None
    r"""The description of the capture."""

    settlement_amount: Annotated[
        OptionalNullable[CaptureResponseSettlementAmount],
        pydantic.Field(alias="settlementAmount"),
    ] = UNSET
    r"""This optional field will contain the approximate amount that will be settled to your account, converted to the
    currency your account is settled in.

    Since the field contains an estimated amount during capture processing, it may change over time. To retrieve
    accurate settlement amounts we recommend using the [List balance transactions endpoint](list-balance-transactions)
    instead.
    """

    metadata: OptionalNullable[Metadata] = UNSET
    r"""Provide any data you like, for example a string or a JSON object. We will save the data alongside the entity. Whenever
    you fetch the entity with our API, we will also include the metadata. You can use up to approximately 1kB.
    """

    shipment_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="shipmentId")
    ] = UNSET
    r"""The unique identifier of the shipment that triggered the creation of this capture, if applicable. For example:
    `shp_gNapNy9qQTUFZYnCrCF7J`.
    """

    settlement_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="settlementId")
    ] = UNSET
    r"""The identifier referring to the settlement this capture was settled with. For example, `stl_BkEjN2eBb`. This field
    is omitted if the capture is not settled (yet).
    """

    @field_serializer("mode")
    def serialize_mode(self, value):
        if isinstance(value, str):
            try:
                return models.Mode(value)
            except ValueError:
                return value
        return value

    @field_serializer("status")
    def serialize_status(self, value):
        if isinstance(value, str):
            try:
                return models.CaptureResponseStatus(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "description",
                "settlementAmount",
                "metadata",
                "shipmentId",
                "settlementId",
            ]
        )
        nullable_fields = set(
            ["amount", "settlementAmount", "metadata", "shipmentId", "settlementId"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    CaptureResponseLinks.model_rebuild()
except NameError:
    pass
try:
    CaptureResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 73e09bd2757a

from __future__ import annotations
from enum import Enum
from mollie import utils


class OrganizationVatRegulation(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""Mollie applies Dutch VAT for merchants based in The Netherlands, British VAT for merchants based in The United
    Kingdom, and shifted VAT for merchants in the European Union.

    The field is not present for merchants residing in other countries.
    """

    DUTCH = "dutch"
    BRITISH = "british"
    SHIFTED = "shifted"
